library jaguar_mongo.interceptor;

import 'dart:async';

import 'package:jaguar/jaguar.dart';
import 'package:mongo_dart/mongo_dart.dart' as mongo;

import 'state.dart';
import 'pool.dart';

class WrapMongoDb implements RouteWrapper<MongoDb> {
  final String mongoUri;

  final int poolSize;

  final mongo.WriteConcern writeConcern;

  final String id;

  final Map<Symbol, MakeParam> makeParams;

  const WrapMongoDb(this.mongoUri,
      {this.writeConcern: mongo.WriteConcern.ACKNOWLEDGED,
      this.poolSize: 10,
      this.id,
      this.makeParams});

  MongoDb createInterceptor() =>
      new MongoDb(mongoUri, poolSize: poolSize, writeConcern: writeConcern);
}

class MongoDb extends Interceptor {
  final String mongoUri;
  final int poolSize;
  final mongo.WriteConcern writeConcern;
  final MongoDbState _state = new MongoDbState();

  MongoDb(this.mongoUri,
      {this.writeConcern: mongo.WriteConcern.ACKNOWLEDGED, this.poolSize: 10});

  static MongoDbState createState() => new MongoDbState();

  Future<mongo.Db> pre() async {
    if (MongoDbState.mongoDbPool[mongoUri] == null) {
      MongoDbState.mongoDbPool[mongoUri] = new MongoDbPool(mongoUri,
          poolSize: poolSize, writeConcern: writeConcern);
    }
    _state.managedConnection =
        await MongoDbState.mongoDbPool[mongoUri].getConnection();
    return _state.managedConnection.conn;
  }

  void post() {
    return MongoDbState.mongoDbPool[mongoUri]
        .releaseConnection(_state.managedConnection);
  }
}
